<?php

declare(strict_types=1);

namespace Drupal\zalo_zns\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\message\Entity\Message;
use Drupal\message_notify\MessageNotifier;
use Drupal\zalo_zns\Helper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Zalo Zns form.
 */
final class ZaloZnsTestForm extends FormBase {

  /**
   * Message notifier.
   *
   * @var \Drupal\message_notify\MessageNotifier
   */
  protected MessageNotifier $messageNotifier;

  /**
   * Zalo ZNS helper.
   *
   * @var \Drupal\zalo_zns\Helper
   */
  protected Helper $zaloZnsHelper;

  /**
   * {@inheritDoc}
   */
  public function __construct(MessageNotifier $messageNotifier, Helper $zaloZnsHelper) {
    $this->messageNotifier = $messageNotifier;
    $this->zaloZnsHelper = $zaloZnsHelper;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('message_notify.sender'),
      $container->get('zalo_zns.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'zalo_zns_zalo_zns_test';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['method'] = [
      '#type' => 'radios',
      '#title' => $this->t('Method'),
      '#description' => $this->t('When select "Message notify",<br>development mode base on config in settings.php and can not override access token.'),
      '#options' => [
        'send_directly' => $this->t('Send directly'),
        'message_notify' => $this->t('Message notify'),
      ],
      '#default_value' => 'send_directly',
    ];
    $form['dev_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Development mode'),
      '#default_value' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="method"]' => ['value' => 'send_directly'],
        ],
      ],
    ];
    $form['phone_no'] = [
      '#type' => 'tel',
      '#title' => $this->t('Phone number'),
      '#required' => TRUE,
    ];
    $form['template_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Template ID'),
      '#description' => $this->t('If Method is "Send directly", it should be ZNS template id from Zalo ZNS.<br>If Method is "Message notify", it should be message template id from Drupal system.'),
      '#required' => TRUE,
    ];
    $form['template_data'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Template data'),
      '#required' => TRUE,
      '#description' => $this->t('One parameter per row, format: key|value<br>If Method is "Send directly", key should be ZNS template parameter from Zalo ZNS.<br>If Method is "Message notify", it should be machine name of message template field from Drupal system.'),
    ];
    $form['override_access_token'] = [
      '#type' => 'textarea',
      '#rows' => 2,
      '#title' => $this->t('Override access token'),
      '#required' => FALSE,
      '#states' => [
        'visible' => [
          ':input[name="method"]' => ['value' => 'send_directly'],
        ],
      ],
    ];
    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Send'),
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $form_state->setRebuild(TRUE);

    $method = $form_state->getValue('method');
    $rawTemplateData = $form_state->getValue('template_data');
    $rawTemplateData = explode(PHP_EOL, $rawTemplateData);
    $templateData = [];
    foreach ($rawTemplateData as $datum) {
      $explodes = explode('|', trim($datum));
      $templateData[reset($explodes)] = end($explodes);
    }
    $phoneNo = $form_state->getValue('phone_no');
    $templateId = $form_state->getValue('template_id');

    if ($method == 'send_directly') {
      // Send request.
      try {
        $response = $this->zaloZnsHelper->sendZnsNotification($form_state->getValue('dev_mode'), $phoneNo, $templateId, $templateData, NULL, $form_state->getValue('override_access_token'));
        $result = $response->getBody()->getContents();
        $this->messenger()
          ->addStatus($this->t('The message has been sent. Data: <br><em>@json</em>', [
            '@json' => $result,
          ]));
      }
      catch (\Exception $exception) {
        $this->messenger()
          ->addError($this->t('The message has not been sent. Message: @message', [
            '@message' => $exception->getMessage(),
          ]));
        $this->messenger()->addError($exception->getMessage());
      }
    }
    elseif ($method == 'message_notify') {
      $message = Message::create(array_merge([
        'template' => $templateId,
        'field_send_to' => $phoneNo,
      ], $templateData));
      $message->save();
      $result = $this->messageNotifier->send($message, [], 'zalo_zns');

      $this->messenger()
        ->addStatus($this->t('The message has been sent. Result: @result', [
          '@result' => $result ? 'Success' : 'Failed',
        ]));
    }
  }

}
